import { Nullable, Optional, Array1, Array2, ReadonlyArray1, ReadonlyArray2, primitive, Int32, CodePoint } from "wiinuk-extensions";
import { Is, ArbitraryCore, Arbitrary, ArrayArbitraryOptions } from "./arbitrary";
export * from "./random";
export * from "./arbitrary";
export interface Show<T> {
    stringify(value: T): string;
}
export declare class ResultBase {
    readonly value: string;
    readonly labels: Set<string> | null;
    constructor(value: string, labels: Set<string> | null);
}
export declare class Failure extends ResultBase {
    protected readonly _failure: null;
    readonly type: string;
}
export declare class Success extends ResultBase {
    protected readonly _success: null;
    readonly type: string;
}
export declare class Exception extends ResultBase {
    readonly error: any;
    protected readonly _exception: null;
    readonly type: string;
    constructor(error: any, value: string, labels: Set<string> | null);
}
export interface TestResultBase<T> {
    readonly show: Show<T>;
    readonly seed: number;
    readonly testCount: number;
}
export interface TestFailureArgs<T> extends TestResultBase<T> {
    readonly shrinkCount: number;
    readonly originalFail: T;
    readonly minFail: T;
}
export interface TestFailure<T> extends TestFailureArgs<T> {
    readonly type: "Failure";
}
export interface TestFailureWithExceptionArgs<T> extends TestFailureArgs<T> {
    readonly error: any;
}
export interface TestFailureWithException<T> extends TestFailureWithExceptionArgs<T> {
    readonly type: "Exception";
}
export interface TestSuccessArgs<T> extends TestResultBase<T> {
}
export interface TestSuccess<T> extends TestSuccessArgs<T> {
    readonly type: "Success";
}
export declare type TestResult<T> = TestFailure<T> | TestFailureWithException<T> | TestSuccess<T>;
export declare namespace TestResult {
    function testFailure<T>(args: TestFailureArgs<T>): TestFailure<T>;
    function testSuccess<T>(args: TestSuccessArgs<T>): TestSuccess<T>;
}
export interface Runner {
    onShrink(shrinkCount: number, maxValue: string, currentValue: string): void;
    onTest(testCount: number, currentValue: string): void;
    onFinish<T>(result: TestResult<T>): void;
}
export declare namespace Runner {
    function fromFunction(log: (message: string) => void): Runner;
    const console: Runner;
}
export interface Config {
    readonly seed?: number;
    readonly maxTest: number;
    readonly startSize: number;
    readonly endSize: number;
    readonly runner?: Runner;
}
export declare namespace Config {
    const defaultValue: Config;
}
export interface Checker<T> extends Arbitrary<T>, Show<T> {
    check(test: (value: T) => any, config?: Config): TestResult<T>;
    array(options: {
        readonly min: 1;
    }): Checker<Array1<T>>;
    array(options: {
        readonly min: 2;
    }): Checker<Array2<T>>;
    array(options?: {
        readonly min?: number;
    }): Checker<T[]>;
    readonlyArray(options: {
        readonly min: 1;
    }): Checker<ReadonlyArray1<T>>;
    readonlyArray(options: {
        readonly min: 2;
    }): Checker<ReadonlyArray2<T>>;
    readonlyArray(options?: {
        readonly min?: number;
    }): Checker<ReadonlyArray<T>>;
    map<U extends T>(convertTo: (value: T) => U): Checker<U>;
    map<U>(convertTo: (value: T) => U, convertFrom: (value: U) => T): Checker<U>;
    filter(predicate: (value: T) => boolean): Checker<T>;
    nullable<A extends {} | undefined>(this: Checker<A>): Checker<Nullable<A>>;
    optional<A extends {} | null>(this: Checker<A>): Checker<Optional<A>>;
}
export declare function fromArbitrary<T>(arbitrary: ArbitraryCore<T>): Checker<T>;
export declare function pure(value: null): Checker<null>;
export declare function pure(value: undefined): Checker<undefined>;
export declare function pure<T extends string | number | boolean>(value: T): Checker<T>;
export declare function pure<T>(value: T): Checker<T>;
export declare function elements<T extends primitive>(value: T, ...values: T[]): Checker<T>;
export declare const number: Checker<number>;
export declare const int32: Checker<Int32>;
export declare const codePoint: Checker<CodePoint>;
export declare function array<T>(arbitrary: ArbitraryCore<T>, options: ArrayArbitraryOptions<1>): Checker<Array1<T>>;
export declare function array<T>(arbitrary: ArbitraryCore<T>, options: ArrayArbitraryOptions<2>): Checker<Array2<T>>;
export declare function array<T>(arbitrary: ArbitraryCore<T>, options?: Partial<ArrayArbitraryOptions<number>>): Checker<T[]>;
export declare const string: Checker<string>;
export declare function interface_<T>(arbitraryMap: {
    [P in keyof T]: ArbitraryCore<T[P]>;
}): Checker<T>;
export declare function tuple<T1>(arbitrary1: ArbitraryCore<T1>): Checker<[T1]>;
export declare function tuple<T1, T2>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>): Checker<[T1, T2]>;
export declare function tuple<T1, T2, T3>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>): Checker<[T1, T2, T3]>;
export declare function tuple<T1, T2, T3, T4>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>): Checker<[T1, T2, T3, T4]>;
export declare function tuple<T1, T2, T3, T4, T5>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>): Checker<[T1, T2, T3, T4, T5]>;
export declare function tuple<T1, T2, T3, T4, T5, T6>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>, arbitrary6: ArbitraryCore<T6>): Checker<[T1, T2, T3, T4, T5, T6]>;
export declare function tuple<T1, T2, T3, T4, T5, T6, T7>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>, arbitrary6: ArbitraryCore<T6>, arbitrary7: ArbitraryCore<T7>): Checker<[T1, T2, T3, T4, T5, T6, T7]>;
export declare function tuple<T1, T2, T3, T4, T5, T6, T7, T8>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>, arbitrary6: ArbitraryCore<T6>, arbitrary7: ArbitraryCore<T7>, arbitrary8: ArbitraryCore<T8>): Checker<[T1, T2, T3, T4, T5, T6, T7, T8]>;
export declare function tuple<T1, T2, T3, T4, T5, T6, T7, T8, T9>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>, arbitrary6: ArbitraryCore<T6>, arbitrary7: ArbitraryCore<T7>, arbitrary8: ArbitraryCore<T8>, arbitrary9: ArbitraryCore<T9>): Checker<[T1, T2, T3, T4, T5, T6, T7, T8, T9]>;
export declare function tuple<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>, arbitrary6: ArbitraryCore<T6>, arbitrary7: ArbitraryCore<T7>, arbitrary8: ArbitraryCore<T8>, arbitrary9: ArbitraryCore<T9>, arbitrary10: ArbitraryCore<T10>): Checker<[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10]>;
export declare function tuple<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>, arbitrary6: ArbitraryCore<T6>, arbitrary7: ArbitraryCore<T7>, arbitrary8: ArbitraryCore<T8>, arbitrary9: ArbitraryCore<T9>, arbitrary10: ArbitraryCore<T10>, arbitrary11: ArbitraryCore<T11>): Checker<[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11]>;
export declare function tuple<T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12>(arbitrary1: ArbitraryCore<T1>, arbitrary2: ArbitraryCore<T2>, arbitrary3: ArbitraryCore<T3>, arbitrary4: ArbitraryCore<T4>, arbitrary5: ArbitraryCore<T5>, arbitrary6: ArbitraryCore<T6>, arbitrary7: ArbitraryCore<T7>, arbitrary8: ArbitraryCore<T8>, arbitrary9: ArbitraryCore<T9>, arbitrary10: ArbitraryCore<T10>, arbitrary11: ArbitraryCore<T11>, arbitrary12: ArbitraryCore<T12>): Checker<[T1, T2, T3, T4, T5, T6, T7, T8, T9, T10, T11, T12]>;
export declare function tuple<T>(arbitrary: ArbitraryCore<T>, ...arbitraries: ArbitraryCore<T>[]): Checker<T[]>;
export declare function sum<T1, T2>(arbitrary1: [ArbitraryCore<T1>, Is<T1 | T2, T1>], arbitrary2: [ArbitraryCore<T2>, Is<T1 | T2, T2>]): Checker<T1 | T2>;
export declare function sum<T1, T2, T3>(arbitrary1: [ArbitraryCore<T1>, Is<T1 | T2 | T3, T1>], arbitrary2: [ArbitraryCore<T2>, Is<T1 | T2 | T3, T2>], arbitrary3: [ArbitraryCore<T3>, Is<T1 | T2 | T3, T3>]): Checker<T1 | T2 | T3>;
export declare function sum<T1, T2, T3, T4>(arbitrary1: [ArbitraryCore<T1>, Is<T1 | T2 | T3 | T4, T1>], arbitrary2: [ArbitraryCore<T2>, Is<T1 | T2 | T3 | T4, T2>], arbitrary3: [ArbitraryCore<T3>, Is<T1 | T2 | T3 | T4, T3>], arbitrary4: [ArbitraryCore<T4>, Is<T1 | T2 | T3 | T4, T4>]): Checker<T1 | T2 | T3 | T4>;
export declare function sum<T1, T2, T3, T4, T5>(arbitrary1: [ArbitraryCore<T1>, Is<T1 | T2 | T3 | T4 | T5, T1>], arbitrary2: [ArbitraryCore<T2>, Is<T1 | T2 | T3 | T4 | T5, T2>], arbitrary3: [ArbitraryCore<T3>, Is<T1 | T2 | T3 | T4 | T5, T3>], arbitrary4: [ArbitraryCore<T4>, Is<T1 | T2 | T3 | T4 | T5, T4>], arbitrary5: [ArbitraryCore<T5>, Is<T1 | T2 | T3 | T4 | T5, T5>]): Checker<T1 | T2 | T3 | T4 | T5>;
export declare function sum<T1, T2, T3, T4, T5, T6>(arbitrary1: [ArbitraryCore<T1>, Is<T1 | T2 | T3 | T4 | T5 | T6, T1>], arbitrary2: [ArbitraryCore<T2>, Is<T1 | T2 | T3 | T4 | T5 | T6, T2>], arbitrary3: [ArbitraryCore<T3>, Is<T1 | T2 | T3 | T4 | T5 | T6, T3>], arbitrary4: [ArbitraryCore<T4>, Is<T1 | T2 | T3 | T4 | T5 | T6, T4>], arbitrary5: [ArbitraryCore<T5>, Is<T1 | T2 | T3 | T4 | T5 | T6, T5>], arbitrary6: [ArbitraryCore<T6>, Is<T1 | T2 | T3 | T4 | T5 | T6, T6>]): Checker<T1 | T2 | T3 | T4 | T5 | T6>;
export declare function sum<T1, T2, T3, T4, T5, T6, T7>(arbitrary1: [ArbitraryCore<T1>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7, T1>], arbitrary2: [ArbitraryCore<T2>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7, T2>], arbitrary3: [ArbitraryCore<T3>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7, T3>], arbitrary4: [ArbitraryCore<T4>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7, T4>], arbitrary5: [ArbitraryCore<T5>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7, T5>], arbitrary6: [ArbitraryCore<T6>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7, T6>], arbitrary7: [ArbitraryCore<T7>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7, T7>]): Checker<T1 | T2 | T3 | T4 | T5 | T6 | T7>;
export declare function sum<T1, T2, T3, T4, T5, T6, T7, T8>(arbitrary1: [ArbitraryCore<T1>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T1>], arbitrary2: [ArbitraryCore<T2>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T2>], arbitrary3: [ArbitraryCore<T3>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T3>], arbitrary4: [ArbitraryCore<T4>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T4>], arbitrary5: [ArbitraryCore<T5>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T5>], arbitrary6: [ArbitraryCore<T6>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T6>], arbitrary7: [ArbitraryCore<T7>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T7>], arbitrary8: [ArbitraryCore<T8>, Is<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8, T8>]): Checker<T1 | T2 | T3 | T4 | T5 | T6 | T7 | T8>;
export declare function sum<T>(arbitrary: [ArbitraryCore<T>, Is<T, T>], ...arbitraries: [ArbitraryCore<T>, Is<T, T>][]): Checker<T>;
export interface ForwardDeclarationChecker<T> extends Checker<T> {
    definition: Checker<T>;
}
export declare function forwardDeclaration<T>(): ForwardDeclarationChecker<T>;
